from infi.shell.commands import commandgroup, command, magic
from infi.shell.shell import get_active_shell

from ..renderer import CSVRenderer

from .. import VERSION
from .. import raw_mode
from .. import debug

@commandgroup
class InfiniboxMagicCommands(object):
    @magic
    @command(
        description="Display client version",
        )
    def version(self, _api):
        return VERSION
    @magic
    @command(
        description="Toggle debug prints",
        )
    def debug(self, _api):
        if debug.is_on():
            debug.off()
        else:
            debug.on()
        return str(debug)
    @magic
    @command(
        description="Toggle raw API output",
        )
    def raw(self, _api):
        raw_mode.toggle()
        return "Raw mode is " + ("on" if raw_mode.is_on() else "off")
    @magic
    @command(
        description="Display name of active user",
        )
    def user(self, _api):
        return _api.get_username()
    @magic
    @command(
        description="Toggle CSV output mode",
        )
    def csv(self, _api):
        shell = get_active_shell()
        current_override = shell.get_renderer_override()
        if current_override is None or not isinstance(current_override, CSVRenderer):
            shell.set_renderer_override(CSVRenderer())
            return "CSV output enabled"
        else:
            shell.set_renderer_override(None)
            return "CSV output disabled"
