from infi.shell.commands import commandgroup, command
from infi.shell.commands.types import Str
from .types import ObjType, Name
from ..exceptions import BadURI
from infi.shell.exceptions import CommandFailed
from collections import OrderedDict

@commandgroup(prefix="metadata")
class MetadataCommands(object):
    def _get_object(self, _api, objtype, objname):
        return ObjType.get_type(objtype).get_by_name(_api, objname)
    @command(
        description="Set metadata on object",
        types=(ObjType, Name, Str, Str),
        substitute=dict(objname="object"),
        )
    def set(self, _api, objtype, objname, key, value):
        obj = self._get_object(_api, objtype, objname)
        data = {key : value}
        _api.post("metadata/{}".format(obj.id), data)
        return "Meta-data set"
    @command(
        description="Get metadata from object",
        types=(ObjType, Name, Str),
        substitute=dict(objname="object"),
        )
    def get(self, _api, objtype, objname, key=None):
        obj = self._get_object(_api, objtype, objname)
        if key is None:
            return [OrderedDict((("key", k), ("value", v))) # Replace OrderedDict by @field_priority
                    for k, v in _api.get("metadata/{}".format(obj.id)).iteritems()]
        try:
            return _api.get(u"metadata/{}/{}".format(obj.id, key))
        except BadURI:
            raise NoSuchKey(key)
    @command(
        description="Remove single metadata key from object",
        types=(ObjType, Name, Str),
        substitute=dict(objname="object"),
        )
    def remove(self, _api, objtype, objname, key):
        obj = self._get_object(_api, objtype, objname)
        _api.delete(u"metadata/{}/{}".format(obj.id, key))
        return "Meta-data removed"
    @command(
        description="Clear all metadata from object",
        types=(ObjType, Name),
        substitute=dict(objname="object"),
        )
    def clear(self, _api, objtype, objname):
        obj = self._get_object(_api, objtype, objname)
        _api.delete("metadata/{}".format(obj.id))
        return "All meta-data cleared"

class NoSuchKey(CommandFailed):
    pass
